@@ -77,15 +77,16 @@ class AutonomousExperimenterGP():
         if init_dataset_size is None and x is None and dataset is None:
             raise Exception("Either provide length of initial data or an inital dataset")
         self.data = gpData(dim, parameter_bounds)
-        if (x is None or y is None) and dataset is None:
+        if  x is None and dataset is None:
             self.data.create_random_dataset(init_dataset_size)
             self.data.dataset = self.instrument_func(self.data.dataset)
-        elif (x is None or y is None) and dataset is not None:
+        elif dataset is not None:
             self.data.inject_dataset(list(np.load(dataset, allow_pickle = True)))
             hyperparameters = self.data.dataset[-1]["hyperparameters"]
         elif x is not None and y is not None:
-            self.data.inject_arrays(x)
-            self.data.dataset = self.instrument_func(self.data.dataset)
+            self.data.dataset = self.data.inject_arrays(x,y=y,v=v)
+        elif x is not None and y is None:
+            self.data.dataset = self.instrument_func(self.data.inject_arrays(x,y=y,v=v))
         else: raise Exception("No viable option for data given!")
         if self.data.nan_in_dataset(): self.data.clean_data_NaN()
         self.x, self.y, self.v, self.t, self.c = self.data.extract_data()
@@ -118,19 +119,24 @@ class AutonomousExperimenterGP():
         tolerance = tol,max_iter = max_iter,
         dask_client = dask_client
         )
+        print("The Autonomous Experimenter started an instance of the asynchronous training.")
         self.async_train = True
 
     def kill_training(self):
         print("async training is being killed")
         if self.async_train: self.gp_optimizer.stop_async_train(self.opt_obj)
+        else: print("no training to be killed")
         self.async_train = False
 
     def kill_client(self):
         if self.async_train: self.gp_optimizer.kill_async_train(self.opt_obj)
 
     def update_hps(self):
-        if self.async_train: self.gp_optimizer.update_hyperparameters(self.opt_obj)
-        print("The Autonomus Experimenter updated the Hyperparameters")
+        print("The autonomous experimenter is trying to update the huperparameters.")
+        if self.async_train:
+            self.gp_optimizer.update_hyperparameters(self.opt_obj)
+            print("The Autonomus Experimenter updated the Hyperparameters")
+        else: print("The autonomous experimenter could not find an instance of asynchronous training. Therefore, no update.")
         print("hps: ", self.gp_optimizer.hyperparameters)
 
     def _init_costs(self,cost_func_params):
@@ -197,12 +203,13 @@ class AutonomousExperimenterGP():
             print("")
             print("")
             print("")
-            print("====================")
-            print("====================")
+            print("==================================")
+            print("==================================")
             print("iteration: ",i)
             print("Run Time: ", time.time() - start_time, "     seconds")
             print("Number of measurements: ", n_measurements)
-            print("====================")
+            print("==================================")
+            print("==================================")
             #ask() for new suggestions
             current_position = self.x[-1]
             print("hps: ",self.gp_optimizer.hyperparameters)
@@ -224,7 +231,7 @@ class AutonomousExperimenterGP():
             error = np.max(np.sqrt(post_var[0]))
             if acq_func_opt_tol_adjust[0]:
                 acq_func_opt_tol = abs(func_evals[0]) * acq_func_opt_tol_adjust[1]
-                print("variance optimization tolerance changed to: ", acq_func_opt_tol)
+                print("acquisition funciton optimization tolerance changed to: ", acq_func_opt_tol)
             print("Next points to be requested: ")
             print(next_measurement_points)
             #update and tell() new data
@@ -237,6 +244,7 @@ class AutonomousExperimenterGP():
             print("Data received")
             print("Checking if data is clean ...")
             if self.data.nan_in_dataset(): self.data.clean_data_NaN()
+            self.data.check_incoming_data()
             print("done")
             #update arrays and the gp_optimizer
             self.x,self.y, self.v, self.t, self.c,vp = self.extract_data()
@@ -244,26 +252,29 @@ class AutonomousExperimenterGP():
             self.tell(self.x, self.y, self.v, vp)
             ###########################
             #train()
-            print("Training ...")
-            if len(self.x) in retrain_async_at:
+            print("++++++++++++++++++++++++++")
+            print("|Training ...            |")
+            print("++++++++++++++++++++++++++")
+            if n_measurements in retrain_async_at:
+                print("    Starting  a new asynchronous training after killing the current one.")
                 self.kill_training()
-                self.opt_obj = self.train_async(pop_size = 10,tol = 1e-6,max_iter = 20,
+                self.train_async(pop_size = 10,tol = 1e-6,max_iter = 100000000,
                                  dask_client = self.training_dask_client)
-            elif len(self.x) in retrain_globally_at:
+            elif n_measurements in retrain_globally_at:
                 self.kill_training()
-                print("Fresh optimization from scratch via global optimization")
+                print("    Fresh optimization from scratch via global optimization")
                 self.train(pop_size = 10,tol = 1e-6,max_iter = 20, method = "global")
-            elif len(self.x) in retrain_locally_at:
+            elif n_measurements in retrain_locally_at:
                 self.kill_training()
-                print("Fresh optimization from scratch via global optimization")
+                print("    Fresh optimization from scratch via global optimization")
                 self.train(pop_size = 10,tol = 1e-6, max_iter = 20, method = "local")
-            elif len(self.x) in retrain_callable_at:
+            elif n_measurements in retrain_callable_at:
                 self.kill_training()
-                print("Fresh optimization from scratch via global optimization")
+                print("    Fresh optimization from scratch via user-defined optimization")
                 self.train(pop_size = 10,tol = 1e-6,max_iter = 20, method = training_opt)
             else:
+                print("    No training in this round but I am trying to update the hyperparameters")
                 self.update_hps()
-                print("No training in this round but I tried to update the hyperparameters")
             ###save some data
             try: np.save('Data_'+ start_date_time, self.data.dataset)
             except Exception as e: print("Data not saved due to ", str(e))
@@ -327,15 +338,16 @@ class AutonomousExperimenterFvGP(AutonomousExperimenterGP):
             raise Exception("Either provide length of initial data or an inital dataset")
         self.data = fvgpData(dim, parameter_bounds,
                 output_number = output_number, output_dim = output_dim)
-        if (x is None or y is None) and dataset is None:
+        if x is None and dataset is None:
             self.data.create_random_dataset(init_dataset_size)
             self.data.dataset = self.instrument_func(self.data.dataset)
-        elif (x is None or y is None) and dataset is not None:
+        elif dataset is not None:
             self.data.inject_dataset(list(np.load(dataset, allow_pickle = True)))
             self.hyperparameters = self.data.dataset[-1]["hyperparameters"]
         elif x is not None and y is not None:
-            self.data.inject_arrays(x)
-            self.data.dataset = self.instrument_func(self.data.dataset)
+            self.data.dataset = self.data.inject_arrays(x,y=y,v=v)
+        elif x is not None and y is None:
+            self.data.dataset = self.instrument_func(self.data.inject_arrays(x,y=y,v=v))
         else: raise Exception("No viable option for data given!")
         if self.data.nan_in_dataset(): self.data.clean_data_NaN()
         self.x, self.y, self.v, self.t, self.c, self.vp = self.data.extract_data()
