@@ -228,6 +228,7 @@ class GPOptimizer(GP):
                 max_iter = max_iter,
                 dask_client = dask_client
                 )
+        print("The GPOptimizer has created an optimization object.")
         return opt_obj
 
 ##############################################################
@@ -282,7 +283,7 @@ class GPOptimizer(GP):
             no input parameters
         """
         try: self.kill_training(opt_obj)
-        except: pass
+        except Exception as e: print("kill not sucessful in GPOptimizer due to: ",str(e))
 
 ##############################################################
     def update_hyperparameters(self, opt_obj):
